@@ -90,7 +90,9 @@ class DependencyParser:
             # Covariant subtyping doesn't work in mypy :(
             return self.parser(lockfile_path, manifest_path)  # type: ignore[return-value]
         except Exception as e:
-            logger.error(f"Failed to parse {lockfile_path} with exception {e}")
+            logger.error(
+                f"Failed to parse {lockfile_path} with exception {type(e).__name__}: {e}"
+            )
             return (
                 [],
                 [
