@@ -50,6 +50,7 @@ from semgrep.engine import EngineType
 from semgrep.error import FATAL_EXIT_CODE
 from semgrep.error import INVALID_API_KEY_EXIT_CODE
 from semgrep.error import MISSING_CONFIG_EXIT_CODE
+from semgrep.error import SemgrepCoreError
 from semgrep.error import SemgrepError
 from semgrep.git import git_check_output
 from semgrep.git import is_git_repo_empty
@@ -64,6 +65,7 @@ from semgrep.parsing_data import ParsingData
 from semgrep.rule import Rule
 from semgrep.rule_match import RuleMatch
 from semgrep.rule_match import RuleMatchMap
+from semgrep.run_scan import AutofixBehavior
 from semgrep.simple_profiling import profiling
 from semgrep.state import get_state
 from semgrep.target_manager import ALL_PRODUCTS
@@ -280,7 +282,7 @@ def ci(
     verbose: bool,
     x_eio: bool,
     x_parmap: bool,
-    x_tr: bool,
+    enable_transitive_reachability: Optional[bool],
     x_pro_naming: bool,
     x_semgrepignore_filename: Optional[str],
     x_no_python_schema_validation: bool,
@@ -293,6 +295,7 @@ def ci(
     partial_config: Optional[Path],
     partial_output: Optional[Path],
     x_group_taint_rules: bool,
+    x_dump_symbol_analysis: bool,
 ) -> None:
     if x_simple_profiling:
         simple_profiling_module.enabled_simple_profiling = True
@@ -312,7 +315,7 @@ def ci(
         )
 
         if trace:
-            logger.debug(f"Trace ID: {state.traces.get_trace_id():x}")
+            logger.verbose(f"Trace ID: {state.traces.get_trace_id():x}")
 
         state.metrics.configure(metrics)
         state.error_handler.configure(suppress_errors)
@@ -404,6 +407,7 @@ def ci(
                 dry_run = True
 
             scan_handler = ScanHandler(
+                enable_transitive_reachability=enable_transitive_reachability,
                 dry_run=dry_run,
                 partial_output=partial_output,
                 dump_scan_id_path=dump_scan_id_path,
@@ -445,6 +449,7 @@ def ci(
                         matches_by_rule=FilteredMatches(kept={}, removed={}),
                         rules=[],
                         targets=set(),
+                        skipped_paths=set(),
                         renamed_targets=set(),
                         ignored_targets=frozenset(),
                         cli_suggested_exit_code=0,  # Inform app that we are exiting with code 0
@@ -685,6 +690,12 @@ def ci(
                 )
             final_baseline_commit = metadata.merge_base_ref
 
+        autofix_behavior = (
+            AutofixBehavior.REPORT
+            if scan_handler and scan_handler.autofix
+            else AutofixBehavior.IGNORE
+        )
+
         semgrep_commands_ci_span.set_attribute(
             "scan.scan_type", "diff" if baseline_commit is not None else "full"
         )
@@ -712,8 +723,8 @@ def ci(
             "exclude": per_product_excludes,
             "exclude_rule": exclude_rule,
             "max_target_bytes": max_target_bytes,
-            "autofix": scan_handler.autofix if scan_handler else False,
-            "dryrun": dry_run,
+            "autofix": autofix_behavior,
+            "write_to_tr_cache": not dry_run,
             # Determine whether or not we will sort ignored matches into the
             # `kept` category of FilteredMatches.
             # If there are multiple outputs and any request to keep_ignores
@@ -736,8 +747,10 @@ def ci(
             "capture_core_stderr": capture_core_stderr,
             "allow_local_builds": allow_local_builds,
             "x_parmap": x_parmap,
-            "x_tr": (
-                scan_handler.transitive_reachability_enabled if scan_handler else x_tr
+            "enable_transitive_reachability": (
+                scan_handler.transitive_reachability_enabled
+                if scan_handler
+                else enable_transitive_reachability
             ),
             "x_pro_naming": x_pro_naming,
             "dump_rule_partitions_params": dump_rule_partitions_params,
@@ -745,10 +758,13 @@ def ci(
             "resolve_all_deps_in_diff_scan": (
                 scan_handler.resolve_all_deps_in_diff_scan if scan_handler else False
             ),
-            "symbol_analysis": scan_handler.symbol_analysis if scan_handler else False,
+            "run_symbol_analysis": scan_handler.symbol_analysis
+            if scan_handler
+            else False,
             "fips_mode": scan_handler.fips_mode if scan_handler else False,
             "semgrepignore_filename": x_semgrepignore_filename,
             "x_group_taint_rules": x_group_taint_rules,
+            "x_dump_symbol_analysis": x_dump_symbol_analysis,
         }
 
         try:
@@ -776,6 +792,8 @@ def ci(
                 _executed_rule_count,
                 _missed_rule_count,
                 all_subprojects,
+                symbol_analysis,
+                sca_symbol_analysis,
             ) = semgrep.run_scan.run_scan(
                 **run_scan_args  # type: ignore
             )
@@ -848,6 +866,8 @@ def ci(
                     _executed_rule_count,
                     _missed_rule_count,
                     _historical_all_subprojects,
+                    _symbol_analysis,
+                    _sca_symbol_analysis,
                 ) = semgrep.run_scan.run_scan(
                     **run_scan_args,  # type: ignore
                     historical_secrets=True,
@@ -952,9 +972,57 @@ def ci(
 
             logger.info("CI scan completed successfully.")
 
+        # Collect paths that failed to scan (timeout, OOM, etc.)
+        skipped_paths: set[Path] = set()
+        for err in semgrep_errors:
+            if isinstance(err, SemgrepCoreError) and err.is_scan_failure():
+                if err.core.location and err.core.location.path:
+                    fpath = Path(err.core.location.path.value)
+                    logger.info(f"Skipping {fpath} due to scan failures. Error: {err}")
+                    skipped_paths.add(fpath)
+
         complete_result: out.CiScanCompleteResponse | None = None
         contributions = semgrep.rpc_call.contributions()
         if scan_handler:
+            # Before we finish the scan, let's upload our symbol analysis if we have it.
+            # This is "scan-adjacent information", which is information we want to save,
+            # but doesn't really have to do with the meat of the scan (findings, etc).
+            # We upload it separately, and outsource to `osemgrep` so we don't duplicate
+            # the implementation.
+            #
+            # This upload takes place before findings are reported to the app via the
+            # /complete endpoint, so that the symbol analysis is available by the time
+            # the dependencies are processed.
+            if scan_handler.symbol_analysis and scan_handler.scan_id and token:
+                # legacy combined symbol analysis
+                #
+                # we can remove this in favor of subproject-based
+                # symbol analysis once we confirm nobody else is
+                # depending on it
+                try:
+                    if symbol_analysis is not None:
+                        semgrep.rpc_call.upload_symbol_analysis(
+                            token, scan_handler.scan_id, symbol_analysis
+                        )
+                except Exception as e:
+                    logger.error(f"Failed to upload symbol analysis: {e}")
+
+                if sca_symbol_analysis is not None:
+                    for subproject_symbol_analysis in sca_symbol_analysis:
+                        manifest = subproject_symbol_analysis.manifest
+                        manifest_path = manifest.path if manifest else None
+
+                        lockfile = subproject_symbol_analysis.lockfile
+                        lockfile_path = lockfile.path if lockfile else None
+
+                        semgrep.rpc_call.upload_subproject_symbol_analysis(
+                            token,
+                            scan_handler.scan_id,
+                            manifest_path,
+                            lockfile_path,
+                            subproject_symbol_analysis.symbol_analysis,
+                        )
+
             with Progress(
                 TextColumn("  {task.description}"),
                 SpinnerColumn(spinner_name="simpleDotsScrolling"),
@@ -964,6 +1032,7 @@ def ci(
                     matches_by_rule=filtered_matches_by_rule,
                     rules=filtered_rules,
                     targets=output_extra.all_targets.targets,
+                    skipped_paths=skipped_paths,
                     renamed_targets=renamed_targets,
                     ignored_targets=ignore_log.unsupported_lang_paths(
                         product=SAST_PRODUCT
@@ -999,24 +1068,6 @@ def ci(
                         else:
                             num_nonblocking_findings += 1
 
-            # Before we finish the scan, let's upload our symbol analysis if we have it.
-            # This is "scan-adjacent information", which is information we want to save,
-            # but doesn't really have to do with the meat of the scan (findings, etc).
-            # We upload it separately, and outsource to `osemgrep` so we don't duplicate
-            # the implementation.
-            if (
-                output_extra.core.symbol_analysis is not None
-                and scan_handler.scan_id
-                and token
-            ):
-                logger.debug(
-                    f"Attempting to upload symbol analysis of {len(output_extra.core.symbol_analysis.value)} symbols"
-                )
-                symbol_analysis = output_extra.core.symbol_analysis
-                semgrep.rpc_call.upload_symbol_analysis(
-                    token, scan_handler.scan_id, symbol_analysis
-                )
-
             if not internal_ci_scan_results:
                 output_handler.output(
                     non_cai_matches_by_rule,
