@@ -54,7 +54,6 @@ from semgrep.core_output import core_matches_to_rule_matches
 from semgrep.core_targets_plan import Plan
 from semgrep.core_targets_plan import Task
 from semgrep.engine import EngineType
-from semgrep.env import Env
 from semgrep.error import SemgrepCoreError
 from semgrep.error import SemgrepError
 from semgrep.error import with_color
@@ -790,10 +789,14 @@ class CoreRunner:
         when it is ran without a proper setup.
 
         The setup that ddprof needs is:
+        - We are running inside SMS
         - DDPROF_ON, DD_ENV, DD_AGENT_HOST, and DD_SERVICE are set
         - CAP_PERFMON is set
         - trace is enabled
         """
+        if not os.environ.get("SEMGREP_MANAGED_SCAN"):
+            return False
+
         trace_enabled = self._trace
         ddprof_on_path = shutil.which("ddprof")
         ddprof_env_vars_set = (
@@ -857,9 +860,8 @@ class CoreRunner:
 
         pyro_caml_on_path = shutil.which("pyro-caml")
 
-        pyro_caml_requested = self._profile or (
-            os.environ.get("SEMGREP_PROFILE", "") != ""
-        )
+        profile_env_set = os.environ.get("SEMGREP_PROFILE", "") != ""
+        pyro_caml_requested = self._profile or (profile_env_set)
         pyro_caml = (pyro_caml_on_path is not None) and pyro_caml_requested
 
         # debug message for pyro-caml
@@ -867,8 +869,8 @@ class CoreRunner:
             reasons = []
             if pyro_caml_on_path is None:
                 reasons.append("pyro-caml is not in PATH")
-            if not self._profile:
-                reasons.append("trace is not enabled")
+            if not (self._profile or profile_env_set):
+                reasons.append("profile is not enabled")
             if reasons:
                 logger.debug(
                     "pyro-caml will not be used for profiling. Reason(s): "
@@ -898,9 +900,8 @@ class CoreRunner:
         Note: this is a list because a target can appear twice (e.g. Java + Generic)
         """
         # The range of target_info is (index into rules x product as json)
-        # Using product as JSON because we want structural equality of products instead of object equality.
         target_info: Dict[
-            Tuple[Target, Language], Tuple[List[int], Set[str]]
+            Tuple[Target, Language], Tuple[List[int], Set[out.Product]]
         ] = collections.defaultdict(lambda: (list(), set()))
 
         unused_rules = []
@@ -921,7 +922,7 @@ class CoreRunner:
                 for target in targets:
                     rules_nums, products = target_info[target, language]
                     rules_nums.append(rule_num)
-                    products.add(rule.product.to_json_string())
+                    products.add(rule.product)
 
             if not some_target:
                 unused_rules.append(rule)
@@ -931,7 +932,7 @@ class CoreRunner:
                 Task(
                     path=target,
                     analyzer=language,
-                    products=tuple(out.Product.from_json_string(x) for x in products),
+                    products=tuple(products),
                     # tuple conversion makes rule_nums hashable, so usable as cache key
                     rule_nums=tuple(rule_nums),
                 )
@@ -1013,27 +1014,13 @@ Could not find the semgrep-core executable. Your Semgrep install is likely corru
 
             use_ddprof = self._check_ddprof_preconditions()
             use_pyro_caml = self._check_pyro_caml_preconditions()
-            env = Env()
-            pyro_caml_address = os.environ.get(
-                "PYRO_CAML_SERVER_ADDRESS",
-                (
-                    # TODO update to non dev url!
-                    "https://pyroscope.dev.private.semgrep.dev"
-                    if env.sms_scan_id is not None and env.sms_scan_id != ""
-                    else "http://localhost:4040"
-                ),
-            )
             cmd = [
                 # bugfix: self._binary_path is an Optional[Path]. The
                 # recommended way to convert a Path to a string is to use the
                 # str function. However, mypy allows the use of str to convert
                 # Optional values to strings. Make sure to check against None
                 # even though mypy won't warn you.
-                *(
-                    ["pyro-caml", "--address", pyro_caml_address]
-                    if use_pyro_caml
-                    else []
-                ),
+                *(["pyro-caml", "-vv"] if use_pyro_caml else []),
                 *(["ddprof"] if use_ddprof else []),
                 str(self._binary_path),
                 "-json",
